import structlog

class LoggingConfig:


    processors=[
                structlog.processors.TimeStamper(fmt="iso",key="timestamp"),
                structlog.processors.format_exc_info,
                structlog.stdlib.add_log_level,
                structlog.processors.format_exc_info,
                structlog.processors.JSONRenderer(),
    ]
    context_class=dict
    logger_factory=structlog.stdlib.LoggerFactory()


"""
Helpr package initialization.
"""

__version__ = "0.4.1"

from .cache import BulkRedisAction, BulkRedisActionType, CacheDatabase, RedisHelper
from .cdn import Cdn
from .common_utils import validate_mobile
from .exceptions import AppException
from .format_response import jsonify_failure, jsonify_success
from .json_encoder import EnhancedJSONEncoder
from .logging import Logger, LoggingContextMiddleware
from .models import (
    Base,
    BulkOperationStatus,
    BulkOperationType,
    BulkUploadLog,
    DeliveryModeEnum,
    InventoryLog,
    InventoryLogStatus,
    ProductInventory,
    StateCodeEnum,
    StatePincodeMap,
    Warehouse,
    WarehouseDeliveryMode,
    WarehouseDeliveryModePincode,
    WarehousePincodeDeliveryTimes,
    WarehouseServiceableState,
    WarehouseStatus,
)
from .s3_helper import generate_presigned_url, upload_to_s3
from .secret_manager import SecretManager
from .token_service import (
    JWTHelper,
    TokenError,
    TokenExpiredError,
    TokenInvalidError,
    TokenMissingError,
)
from .decorators import session_required, auth_check_optional, auth_check_required, configure_auth

__all__ = [
    "validate_mobile",
    "AppException",
    "jsonify_success",
    "jsonify_failure",
    "SecretManager",
    "RedisHelper",
    "CacheDatabase",
    "BulkRedisAction",
    "BulkRedisActionType",
    "JWTHelper",
    "TokenError",
    "TokenMissingError",
    "TokenExpiredError",
    "TokenInvalidError",
    "Cdn",
    "Logger",
    "LoggingContextMiddleware",
    "upload_to_s3",
    "generate_presigned_url",
    "Base",
    "WarehouseStatus",
    "DeliveryModeEnum",
    "StateCodeEnum",
    "Warehouse",
    "StatePincodeMap",
    "WarehouseDeliveryMode",
    "WarehouseDeliveryModePincode",
    "WarehouseServiceableState",
    "WarehousePincodeDeliveryTimes",
    "BulkUploadLog",
    "BulkOperationType",
    "BulkOperationStatus",
    "ProductInventory",
    "InventoryLog",
    "InventoryLogStatus",
    "EnhancedJSONEncoder",
    "session_required",
    "auth_check_optional",
    "auth_check_required",
    "configure_auth",
]



from .base import Base
from .warehouse import (
    WarehouseStatus,
    DeliveryModeEnum,
    StateCodeEnum,
    Warehouse,
    StatePincodeMap,
    WarehouseDeliveryMode,
    WarehouseDeliveryModePincode,
    WarehouseServiceableState,
    WarehousePincodeDeliveryTimes,
    BulkUploadLog,
    BulkOperationType,
    BulkOperationStatus,
    ProductInventory,
    InventoryLog,
    InventoryLogStatus
)

__all__ = [
    "Base",
    "WarehouseStatus",
    "DeliveryModeEnum", 
    "StateCodeEnum",
    "Warehouse",
    "StatePincodeMap",
    "WarehouseDeliveryMode",
    "WarehouseDeliveryModePincode",
    "WarehouseServiceableState",
    "WarehousePincodeDeliveryTimes",
    "BulkUploadLog",
    "BulkOperationType", 
    "BulkOperationStatus",
    "ProductInventory",
    "InventoryLog",
    "InventoryLogStatus"
]


import boto3
from fastapi import HTTPException, UploadFile
from botocore.exceptions import BotoCoreError, ClientError

def is_bucket_public(bucket_name: str, region: str) -> bool:
    """
    Check if an S3 bucket is public using bucket policy status.

    Args:
        bucket_name (str): S3 bucket name.
        region (str): AWS region.

    Returns:
        bool: True if bucket is public, False otherwise.
    """
    s3_client = boto3.client("s3", region_name=region)
    try:
        result = s3_client.get_bucket_policy_status(Bucket=bucket_name)
        return result["PolicyStatus"]["IsPublic"]
    except ClientError as e:
        # If bucket has no policy, it defaults to private
        if e.response["Error"]["Code"] == "NoSuchBucketPolicy":
            return False
        print(f"Error checking bucket policy status: {e}")
        return False

def upload_to_s3(
    file: UploadFile,
    key: str,
    bucket_name: str,
    region: str,
    expiry_seconds: int = 3600
) -> str:
    """
    Upload a file to AWS S3 and return appropriate URL based on bucket visibility.

    Args:
        file (UploadFile): File to upload (from FastAPI UploadFile).
        key (str): S3 object key (path + filename).
        bucket_name (str): S3 bucket name.
        region (str): AWS region name.
        expiry_seconds (int): Expiry time for presigned URL if bucket is private.

    Returns:
        str: Public URL (if bucket is public) or Pre-signed URL (if bucket is private).
    """
    s3_client = boto3.client("s3", region_name=region)

    try:
        s3_client.upload_fileobj(
            file.file,
            bucket_name,
            key,
            ExtraArgs={"ContentType": file.content_type}
        )
    except ClientError as e:
        # AWS client-side error
        raise HTTPException(
            status_code=500,
            detail=f"Failed to upload file to S3: {e.response['Error']['Message']}"
        )
    except BotoCoreError as e:
        # AWS core error
        raise HTTPException(
            status_code=500,
            detail=f"Unexpected AWS error during upload: {str(e)}"
        )
    except Exception as e:
        # Other errors
        raise HTTPException(
            status_code=500,
            detail=f"Unexpected error during upload: {str(e)}"
        )

    # Decide which URL to return based on bucket visibility
    if is_bucket_public(bucket_name, region):
        return f"https://{bucket_name}.s3.{region}.amazonaws.com/{key}"
    else:
        return generate_presigned_url(
            key=key,
            bucket_name=bucket_name,
            region=region,
            expiry_seconds=expiry_seconds
        )

def generate_presigned_url(
    key: str,
    bucket_name: str,
    region: str,
    expiry_seconds: int = 3600
) -> str:
    """
    Generate a pre-signed URL for an S3 object.

    Args:
        key (str): S3 object key.
        bucket_name (str): S3 bucket name.
        region (str): AWS region.
        expiry_seconds (int): Expiry time in seconds (default 1 hour).

    Returns:
        str: Pre-signed URL.
    """
    s3_client = boto3.client("s3", region_name=region)
    try:
        return s3_client.generate_presigned_url(
            "get_object",
            Params={"Bucket": bucket_name, "Key": key},
            ExpiresIn=expiry_seconds
        )
    except Exception as e:
        raise HTTPException(
            status_code=500,
            detail=f"Failed to generate pre-signed URL: {str(e)}"
        )


from ..base import Base
from .enums import WarehouseStatus, DeliveryModeEnum, StateCodeEnum
from .warehouse import Warehouse
from .state_pincode_map import StatePincodeMap
from .warehouse_delivery_mode import WarehouseDeliveryMode
from .warehouse_delivery_mode_pincodes import WarehouseDeliveryModePincode
from .warehouse_servicable_states import WarehouseServiceableState
from .warehouse_pincode_delivery_times import WarehousePincodeDeliveryTimes
from .bulk_upload_log import BulkUploadLog, BulkOperationType, BulkOperationStatus
from .inventory import ProductInventory
from .inventory_log import InventoryLog, InventoryLogStatus

__all__ = [
    "Base",
    "WarehouseStatus",
    "DeliveryModeEnum", 
    "StateCodeEnum",
    "Warehouse",
    "StatePincodeMap",
    "WarehouseDeliveryMode",
    "WarehouseDeliveryModePincode",
    "WarehouseServiceableState",
    "WarehousePincodeDeliveryTimes",
    "BulkUploadLog",
    "BulkOperationType",
    "BulkOperationStatus",
    "ProductInventory",
    "InventoryLog",
    "InventoryLogStatus"
]


from functools import wraps
from fastapi import Request
from fastapi.responses import JSONResponse
from .session_cache import SessionCache
from helpr.format_response import jsonify_failure, jsonify_success
from helpr.token_service import JWTHelper, TokenMissingError, TokenExpiredError, TokenInvalidError
from typing import Dict, Any
from helpr.secret_manager import JWTSigningKeyProvider

# Global key provider instance
_global_key_provider = None

def configure_auth(key_provider: JWTSigningKeyProvider):
    """Configure global key provider for all auth decorators"""
    global _global_key_provider
    _global_key_provider = key_provider

def get_global_key_provider() -> JWTSigningKeyProvider:
    """Get the configured key provider"""
    if _global_key_provider is None:
        raise ValueError("Auth not configured. Call configure_auth() first.")
    return _global_key_provider



def get_token(request: Request) -> Dict[str, Any]:
    """Get and verify JWT token from request."""
    token = request.headers.get("Authorization", "").replace("Bearer ", "")
    
    key_provider = get_global_key_provider()
    
    jwt_helper = JWTHelper(key_provider=key_provider)
    try:
        decoded_token = jwt_helper.verify_token(token=token)
        return decoded_token
    except TokenMissingError:
        raise jsonify_failure(status_code=401, message="Authorization token is missing")
    except TokenExpiredError:
        raise jsonify_failure(status_code=401, message="Authorization token has expired")
    except TokenInvalidError as e:
        raise jsonify_failure(status_code=401, message=f"Invalid authorization token: {str(e)}")
    except ValueError as ve:
        raise jsonify_failure(status_code=500, message=f"Configuration Error: {str(ve)}")
    except Exception as e:
        raise jsonify_failure(status_code=500, message=f"Error in get_token: {str(e)}")
    

def auth_check_optional(f):
    @wraps(f)
    async def decorated_function(request: Request, *args, **kwargs):
        # Initialize user state as None by default
        request.state.user_id = None
        request.state.medusa_user_id = None
        
        # Only try to get token if Authorization header is present and not empty
        auth_header = request.headers.get("Authorization", "")
        token = auth_header.replace("Bearer ", "").strip()
        
        if token:  # Check if token exists and is not empty
            try:
                key_provider = get_global_key_provider()
                jwt_helper = JWTHelper(key_provider=key_provider)
                decoded_token = jwt_helper.verify_token(token=token)
                
                # Attach the user id to the request state
                request.state.user_id = decoded_token.get('sub')
                request.state.medusa_user_id = decoded_token.get('alt_sub')

            except TokenExpiredError:
                raise jsonify_failure(status_code=401, message="Authorization token has expired")

            except (TokenMissingError, TokenInvalidError, ValueError, Exception):
                # For optional auth, we silently continue without setting user info
                # User state remains None as initialized above
                pass
                
        # Call the original function with the updated request
        result = await f(request, *args, **kwargs)
        return result
    return decorated_function

def auth_check_required(f):
    @wraps(f)
    async def decorated_function(request: Request, *args, **kwargs):
        # Get token using global key provider
        decoded_token = get_token(request)
        
        # Attach the user id to the request state
        request.state.user_id = decoded_token.get('sub')
        request.state.medusa_user_id = decoded_token.get('alt_sub')
        
        # Call the original function with the updated request
        result = await f(request, *args, **kwargs)
        return result
    return decorated_function


def session_required(f):
    @wraps(f)
    def decorated_function(request: Request, *args, **kwargs):
        session_visit_id = request.headers.get('X-CLY-SESSION-IDENTIFIER')
        if session_visit_id:    
            session_visit_id = session_visit_id.split("#")
        session_id = session_visit_id[0] if session_visit_id and len(session_visit_id)>0 else None        
        if not session_id or not SessionCache.validate_session_id(session_id):
            client_id = request.headers.get('X-CLY-CLIENT-IDENTIFIER', None)            
            session_id = SessionCache.create_session_id(client_id)
        
        request.state.session_id = session_id
        user_id = getattr(request.state, 'user_id', None)

        try:
            request.state.session_cache = SessionCache(session_id=session_id, user_id = user_id)
            visit_id, visit_count = request.state.session_cache.init_if_not_exists(client_id=request.headers.get('X-CLY-CLIENT-IDENTIFIER', None))
            request.state.visit_id = visit_id
            request.state.visit_count = visit_count
        except ValueError as e:
            return jsonify_failure(message=str(e), status_code=401)

        # Call the original function
        result = f(request, *args, **kwargs)

        # Handle FastAPI response formatting
        if isinstance(result, dict):
            response = JSONResponse(content=result)
        elif isinstance(result, JSONResponse):
            response = result
        else:
            response = result

        # Add session headers to response
        if hasattr(response, 'headers'):
            response.headers['X-CLY-SESSION-IDENTIFIER'] = session_id + "#" + visit_id
            response.headers['X-CLY-VISIT-NUMBER'] = str(visit_count)
        
        return response

    return decorated_function